"use strict";
/**
 * Viewport Utils
 *
 * @description A set of viewport utils
 * @author      Tony Samperi, tonysamperi.github.io
 * @version     2.0.0
 * @date        2019-10-03
 */
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var lodash_defaults_1 = require("lodash.defaults");
var js_viewport_utils_model_1 = require("./js-viewport-utils.model");
__export(require("./js-viewport-utils.model"));
var jsViewportUtils = (function () {
    // IE 8 and lower fail this
    var canUseWindowDimensions = typeof window !== "undefined" && window.innerHeight !== undefined;
    var viewportUtils = {
        /**
         * Determines whether an element is within the viewport
         * @param  {HTMLElement|JQuery<HTMLElement>}  elem       DOM Element (required)
         * @param  {JsViewportSettings}  options    Optional settings
         * @return {Boolean}            Whether the element was completely within the viewport
         */
        inViewport: function (elem, options) {
            var _a;
            if (!elem || typeof elem !== "object" || elem.nodeType !== 1) {
                throw new Error("First argument must be an element!");
            }
            var containerBoundingRect;
            var containerScrollTop;
            var containerScrollLeft;
            var elemBoundingRect;
            var isWindow;
            var scrollBarWidths = [0, 0];
            var sidesRegex = /^TOP$|^RIGHT$|^BOTTOM$|^LEFT$|^ALL$/;
            // TMP VARS
            var i, result = !1;
            // Handle jQuery
            if (typeof jQuery !== "undefined") {
                if (elem.jquery) {
                    elem = elem.get(0);
                }
                // Extract the DOM node from a jQuery collection
                if (!!options && options.container.jquery) {
                    options.container = options.container.get(0);
                }
            }
            // Build configuration from defaults and user-provided settings and metadata
            var config = lodash_defaults_1.default({}, options || {}, js_viewport_utils_model_1.jsViewportDefaults);
            // Use the window as the container if the user specified the body or a non-element
            if (config.container === document.body || config.container.nodeType !== 1) {
                config.container = window;
            }
            // Get the element's bounding rectangle with respect to the viewport
            elemBoundingRect = elem.getBoundingClientRect();
            isWindow = (config.container === window);
            var isSideIn = (_a = {},
                _a[js_viewport_utils_model_1.JsViewportSides.TOP] = function () {
                    if (isWindow) {
                        return (elemBoundingRect.top >= config.top);
                    }
                    else {
                        return (elemBoundingRect.top >= containerScrollTop - (containerScrollTop - containerBoundingRect.top) + config.top);
                    }
                },
                _a[js_viewport_utils_model_1.JsViewportSides.RIGHT] = function () {
                    if (isWindow) {
                        return (elemBoundingRect.right <= (containerBoundingRect.right + containerScrollLeft) - config.right);
                    }
                    else {
                        return (elemBoundingRect.right <= containerBoundingRect.right - scrollBarWidths[0] - config.right);
                    }
                },
                _a[js_viewport_utils_model_1.JsViewportSides.BOTTOM] = function () {
                    var containerHeight = 0;
                    if (isWindow) {
                        if (canUseWindowDimensions) {
                            containerHeight = config.container.innerHeight;
                        }
                        else if (document && document.documentElement) {
                            containerHeight = document.documentElement.clientHeight;
                        }
                    }
                    else {
                        containerHeight = containerBoundingRect.bottom;
                    }
                    return (elemBoundingRect.bottom <= containerHeight - scrollBarWidths[1] - config.bottom);
                },
                _a[js_viewport_utils_model_1.JsViewportSides.LEFT] = function () {
                    if (isWindow) {
                        return (elemBoundingRect.left >= config.left);
                    }
                    else {
                        return (elemBoundingRect.left >= containerScrollLeft - (containerScrollLeft - containerBoundingRect.left) + config.left);
                    }
                },
                // Element is within all four boundaries
                _a[js_viewport_utils_model_1.JsViewportSides.ALL] = function () {
                    return (isSideIn.TOP() && isSideIn.BOTTOM() && isSideIn.LEFT() && isSideIn.RIGHT());
                },
                _a);
            // Get viewport dimensions and offsets
            if (isWindow) {
                containerBoundingRect = document.documentElement.getBoundingClientRect();
                containerScrollTop = document.body.scrollTop;
                containerScrollLeft = window.scrollX || document.body.scrollLeft;
            }
            else {
                containerBoundingRect = config.container.getBoundingClientRect();
                containerScrollTop = config.container.scrollTop;
                containerScrollLeft = config.container.scrollLeft;
            }
            if (containerScrollLeft) {
                scrollBarWidths[0] = 18;
            }
            if (containerScrollTop) {
                scrollBarWidths[1] = 16;
            }
            // Loop through all of the sides
            var sides = config.sides;
            i = sides.length;
            while (i--) {
                // Test the element against each side of the viewport that was requested
                if (sidesRegex.test(sides[i])) {
                    if (isSideIn[sides[i]]()) {
                        result = !0;
                    }
                    else {
                        result = !1;
                        break;
                    }
                }
            }
            return result;
        },
        inViewportTop: function (element) {
            return viewportUtils.inViewport(element, { sides: [js_viewport_utils_model_1.JsViewportSides.TOP] });
        },
        inViewportRight: function (element) {
            return viewportUtils.inViewport(element, { sides: [js_viewport_utils_model_1.JsViewportSides.RIGHT] });
        },
        inViewportBottom: function (element) {
            return viewportUtils.inViewport(element, { sides: [js_viewport_utils_model_1.JsViewportSides.BOTTOM] });
        },
        inViewportLeft: function (element) {
            return viewportUtils.inViewport(element, { sides: [js_viewport_utils_model_1.JsViewportSides.LEFT] });
        }
    };
    return viewportUtils;
})();
exports.inViewport = jsViewportUtils.inViewport;
exports.inViewportTop = jsViewportUtils.inViewportTop;
exports.inViewportRight = jsViewportUtils.inViewportRight;
exports.inViewportBottom = jsViewportUtils.inViewportBottom;
exports.inViewportLeft = jsViewportUtils.inViewportLeft;
//# sourceMappingURL=js-viewport-utils.js.map